#ifndef __XMLIO_CGroupTemplate_impl__
#define __XMLIO_CGroupTemplate_impl__

namespace xmlioserver
{
   using namespace tree;

   /// ////////////////////// Définitions ////////////////////// ///

   template <class U, class V, class W>
      CGroupTemplate<U, V, W>::CGroupTemplate(void)
         : CObjectTemplate<V>() //, V()
         , childMap(), childList()
         , groupMap(), groupList()
   { /* Ne rien faire de plus */ }

   template <class U, class V, class W>
      CGroupTemplate<U, V, W>::CGroupTemplate(const StdString & id)
         : CObjectTemplate<V>(id) //, V()
         , childMap(), childList()
         , groupMap(), groupList()
   { /* Ne rien faire de plus */ }

   template <class U, class V, class W>
      CGroupTemplate<U, V, W>::~CGroupTemplate(void)
   { /* Ne rien faire de plus */ }

   template <class U, class V, class W>
      StdString CGroupTemplate<U, V, W>::toString(void) const
   {
      StdOStringStream oss;
      StdString name = (this->getId().compare(V::GetDefName()) != 0)
                     ? V::GetName()
                     : V::GetDefName();

      oss << "< " << name << " ";
      if (this->hasId() && (this->getId().compare(V::GetDefName()) != 0))
         oss << " id=\"" << this->getId() << "\" ";
         
      if (this->hasChild())
      {
         oss << SuperClassAttribute::toString() << ">" << std::endl;
         
         typename std::vector<boost::shared_ptr<V> >::const_iterator 
            itg = this->groupList.begin(), endg = this->groupList.end();
         typename std::vector<boost::shared_ptr<U> >::const_iterator 
            itc = this->childList.begin(), endc = this->childList.end();
            
         for (; itg != endg; itg++)
         { 
            boost::shared_ptr<V> group = *itg;
            oss << *group << std::endl;
         }
            
         for (; itc != endc; itc++)
         { 
            boost::shared_ptr<U> child = *itc;
            oss << *child << std::endl;
         }
            
         oss << "</ " << name << " >";
      }
      else
      {
         oss << SuperClassAttribute::toString() << "/>";
      }
      return (oss.str());
   }

   template <class U, class V, class W>
      void CGroupTemplate<U, V, W>::fromString(const StdString & str)
   { 
      ERROR("CGroupTemplate<U, V, W>::toString(void)",
            << "[ str = " << str << "] Not implemented yet !");
   }

   template <class U, class V, class W>
      StdString CGroupTemplate<U, V, W>::GetName(void)
   { 
      return (U::GetName().append("_group")); 
   }

   template <class U, class V, class W>
      StdString CGroupTemplate<U, V, W>::GetDefName(void)
   { 
      return (U::GetName().append("_definition")); 
   }

   template <class U, class V, class W>
      const std::vector<boost::shared_ptr<U> >&
         CGroupTemplate<U, V, W>::getChildList(void) const
   { 
      return (this->childList); 
   }

   template <class U, class V, class W>
      const xios_map<StdString, boost::shared_ptr<V> >&
         CGroupTemplate<U, V, W>::getGroupMap(void) const
   { 
      return (this->groupList);
   }

   template <class U, class V, class W>
      bool CGroupTemplate<U, V, W>::hasChild(void) const
   { 
      return ((groupList.size() + childList.size()) > 0); 
   }

   template <class U, class V, class W>
      void CGroupTemplate<U, V, W>::parse(xml::CXMLNode & node)
   { 
      this->parse(node, true); 
   }
   
   template <class U, class V, class W>
      void CGroupTemplate<U, V, W>::solveDescInheritance(const CAttributeMap * const parent)
   {
      if (parent != NULL)
         SuperClassAttribute::setAttributes(parent);
         
      typename std::vector<boost::shared_ptr<U> >::const_iterator 
         itc = this->childList.begin(), endc = this->childList.end();
      typename std::vector<boost::shared_ptr<V> >::const_iterator 
         itg = this->groupList.begin(), endg = this->groupList.end();
             
      for (; itc != endc; itc++)
      { 
         boost::shared_ptr<U> child = *itc;
         child->solveDescInheritance(this);
      }
            
      for (; itg != endg; itg++)
      { 
         boost::shared_ptr<V> group = *itg;
         group->solveRefInheritance();
         group->solveDescInheritance(this);
      }
   }

   template <class U, class V, class W>
      void CGroupTemplate<U, V, W>::getAllChildren(std::vector<boost::shared_ptr<U> > & allc) const
   {
      allc.insert (allc.end(), childList.begin(), childList.end());
      typename std::vector<boost::shared_ptr<V> >::const_iterator 
         itg = this->groupList.begin(), endg = this->groupList.end();
         
      for (; itg != endg; itg++)
      { 
         boost::shared_ptr<V> group = *itg;
         group->getAllChildren(allc);
      }
   }

   template <class U, class V, class W>
      std::vector<boost::shared_ptr<U> > CGroupTemplate<U, V, W>::getAllChildren(void) const
   { 
      std::vector<boost::shared_ptr<U> > allc;
      this->getAllChildren(allc);
      return (allc);
   }

   template <class U, class V, class W>
      void CGroupTemplate<U, V, W>::solveRefInheritance(void)
   { /* Ne rien faire de plus */ }

} // namespace xmlioserver


#endif // __XMLIO_CGroupTemplate_impl__
