// XmlIOServer
#include "xmlioserver.hpp"

#include "attribute_template_impl.hpp"
#include "group_template_impl.hpp"

#include "fake.hpp"

// Point d'entrée du programme principal
int main(int argc, char ** argv, char ** UNUSED(env))
{
   try
   {
      comm::CMPIManager::Initialise(&argc, &argv); // < seulement en mode connecté
      
      // Création d'un contexte
      boost::shared_ptr<CContext> contxt = CTreeManager::CreateContext("mon_context");
      
      
      boost::shared_ptr<CGridGroup>   grid_def   = 
         CObjectFactory::GetObject<CGridGroup>(StdString("grid_definition"));
      boost::shared_ptr<CAxisGroup>   axis_def   = 
         CObjectFactory::GetObject<CAxisGroup>(StdString("axis_definition"));
      boost::shared_ptr<CDomainGroup> domain_def = 
         CObjectFactory::GetObject<CDomainGroup>(StdString("domain_definition"));
      
      // Ajout d'une grille, d'un axe et d'un domaine.
      boost::shared_ptr<CGrid>   grid   = 
         CGroupFactory::CreateChild(grid_def  , StdString("ma_grille"));
      boost::shared_ptr<CAxis>   axis   = 
         CGroupFactory::CreateChild(axis_def  , StdString("mon_axe"));
      boost::shared_ptr<CDomain> domain = 
         CGroupFactory::CreateChild(domain_def, StdString("mon_domaine"));
      
      // Définition des attributs de la grille.
      grid->domain_ref.setValue(StdString("mon_domaine"));
      grid->axis_ref.setValue(StdString("mon_axe"));
      
      // Définition des attributs de l'axe.
      ARRAY_CREATE(zvalue, double, 1, [100]);
           
      axis->size.setValue(zvalue->num_elements());
      axis->zvalue.setValue(zvalue);
      
      // Définition des attributs du domaine l'axe.
      ARRAY_CREATE(latvalue, double, 1, [200]);
      ARRAY_CREATE(lonvalue, double, 1, [200]);
      
      domain->ni_glo.setValue(40);
      domain->nj_glo.setValue(40);
      
      domain->ibegin.setValue(1);
      domain->ni.setValue(20);      
      domain->jbegin.setValue(1);
      domain->nj.setValue(10);
      
      domain->data_dim.setValue(2);
      
      domain->lonvalue.setValue(lonvalue);
      domain->latvalue.setValue(latvalue);
      
      // Résolution
      grid->solveReference();
      
      // Sortie de l'arborescence xml sous forme de fichier.
      CTreeManager::PrintTreeToFile("data/def/test.xml");
      
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
     
      
      /*CXIOSManager::Initialise(CXIOSManager::CLIENT, &argc, &argv);
      CXIOSManager::AddClient("nemo",
                              CMPIManager::GetCommSize(CMPIManager::GetCommWorld()),
                              CMPIManager::GetCommSize(CMPIManager::GetCommWorld()),
                              &nemo_fake_entry);
      CXIOSManager::RunClient();
      CXIOSManager::Finalize();*/

      comm::CMPIManager::Finalize(); // < seulement en mode connecté
      
      //CXIOSManager::AddClient("orchidee", 1, 1, &orchidee_fake_entry);
      //CXIOSManager::AddClient("lmdz"    , 2, 2, &lmdz_fake_entry);
      //CXIOSManager::RunClientServer();

   }
   catch (CException & exc)
   {
      std::cerr << exc.getMessage() << std::endl;
      CMPIManager::Finalize();
      return (EXIT_FAILURE);
   }
   return (EXIT_SUCCESS);
}

