#include "tree_manager.hpp"

#include "attribute_template_impl.hpp"
#include "object_template_impl.hpp"
#include "group_template_impl.hpp"

namespace xmlioserver
{
   namespace tree
   {
      /// ////////////////////// Définitions ////////////////////// ///

      void CTreeManager::SetCurrentContextId(const StdString & id)
      {
         CObjectFactory::SetCurrentContextId(id);
         CGroupFactory::SetCurrentContextId(id);
      }

      boost::shared_ptr<CContext> CTreeManager::CreateContext(const StdString & id)
      {
         boost::shared_ptr<CContextGroup> group_context = CContext::GetContextGroup();
         CTreeManager::SetCurrentContextId(id);
         
         boost::shared_ptr<tree::CContext> context =
               CObjectFactory::CreateObject<tree::CContext>(id);
         CGroupFactory::AddChild(group_context, context);

#define DECLARE_NODE(Name_, name_) \
   CObjectFactory::CreateObject<C##Name_##Definition>(C##Name_##Definition::GetDefName());
#define DECLARE_NODE_PAR(Name_, name_)
#include "../config/node_type.conf"

         return (context);
   }

      //--------------------------------------------------------------
      void CTreeManager::PrintTreeToFile(const StdString & path)
      {
         StdOFStream ofs(path.c_str());
         CTreeManager::PrintTreeToStream(ofs);
         ofs.close();
      }

      void CTreeManager::PrintTreeToString(StdString & content)
      {
         StdOStringStream ostrstream;
         tree::CContext::ShowTree(ostrstream);
         content.assign(CIndentedXml::Indented(ostrstream.str()));
      }

      void CTreeManager::PrintTreeToStream(StdOStream & out)
      {
         StdOStringStream ostrstream;
         tree::CContext::ShowTree(ostrstream);
         out << CIndentedXml::Indented(ostrstream.str());
      }

      //--------------------------------------------------------------

      void CTreeManager::ParseFile(const StdString & filename)
      { xml::CXMLParser::ParseFile(filename); }

      void CTreeManager::ParseString(const StdString & xmlContent)
      { xml::CXMLParser::ParseString(xmlContent); }

      void CTreeManager::ParseStream(StdIStream & stream)
      { xml::CXMLParser::ParseStream(stream); }

      ///-------------------------------------------------------------

   } // namespace tree
} // namespace xmlioserver
