#ifndef __XMLIO_CDomain__
#define __XMLIO_CDomain__

/// xmlioserver headers ///
#include "xmlioserver_spl.hpp"
#include "group_factory.hpp"
#include "mask.hpp"

#include "declare_group.hpp"

namespace xmlioserver {
namespace tree {
   
   /// ////////////////////// Déclarations ////////////////////// ///

   class CDomainGroup;
   class CDomainAttributes;
   class CDomain;

   ///--------------------------------------------------------------

   // Declare/Define CDomainAttribute
   BEGIN_DECLARE_ATTRIBUTE_MAP(CDomain)
   #include "domain_attribute.conf"
   END_DECLARE_ATTRIBUTE_MAP(CDomain)

   ///--------------------------------------------------------------

   class CDomain
      : public CObjectTemplate<CDomain>
      , public CDomainAttributes
   {
         /// typedef ///
         typedef CObjectTemplate<CDomain>   SuperClass;
         typedef CDomainAttributes SuperClassAttribute;

      public :

         typedef CDomainAttributes RelAttributes;
         typedef CDomainGroup      RelGroup;

         /// Constructeurs ///
         CDomain(void);
         explicit CDomain(const StdString & id);
         CDomain(const CDomain & domain);       // Not implemented yet.
         CDomain(const CDomain * const domain); // Not implemented yet.

         /// Vérifications ///
         void checkAttributes(void);

      private :

         virtual void checkGlobalDomain(void);

         virtual void checkLocalIDomain(void);
         virtual void checkLocalJDomain(void);

         virtual void checkMask(void);
         virtual void checkDomainData(void);
         virtual void checkCompression(void);

         virtual void completeMask(void);

      public :

         /// Accesseurs ///
         boost::shared_ptr<CMask> getLocalMask(void) const;
         const std::set<StdString> & getRelFiles(void) const;

         /// Test ///
         bool IsWritten(const StdString & filename) const;

         /// Mutateur ///
         void addRelFile(const StdString & filename);

         /// Destructeur ///
         virtual ~CDomain(void);

         /// Accesseurs statiques ///
         static StdString GetName(void);
         static StdString GetDefName(void);
         
         static ENodeType GetType(void);

       private :

         /// Proriétés protégées ///
         bool isChecked;
         boost::shared_ptr<CMask> local_mask;
         std::set<StdString> relFiles;

   }; // class CDomain

   ///--------------------------------------------------------------

   // Declare/Define CDomainGroup and CDomainDefinition
   DECLARE_GROUP(CDomain);

   ///--------------------------------------------------------------

} // namespace tree
} // namespace xmlioserver

#endif // __XMLIO_CDomain__
